@@ -166,8 +166,14 @@ malloc(size_t size) noexcept
 {
     assert(hooks::malloc);
 
-    void* ptr = hooks::malloc(size);
-    tracking_api::Tracker::trackAllocation(ptr, size, hooks::Allocator::MALLOC);
+    void* ptr;
+    {
+        tracking_api::RecursionGuard guard;
+        ptr = hooks::malloc(size);
+    }
+    if (ptr) {
+        tracking_api::Tracker::trackAllocation(ptr, size, hooks::Allocator::MALLOC);
+    }
     return ptr;
 }
 
@@ -182,7 +188,10 @@ free(void* ptr) noexcept
         tracking_api::Tracker::trackDeallocation(ptr, 0, hooks::Allocator::FREE);
     }
 
-    hooks::free(ptr);
+    {
+        tracking_api::RecursionGuard guard;
+        hooks::free(ptr);
+    }
 }
 
 void*
@@ -190,7 +199,11 @@ realloc(void* ptr, size_t size) noexcept
 {
     assert(hooks::realloc);
 
-    void* ret = hooks::realloc(ptr, size);
+    void* ret;
+    {
+        tracking_api::RecursionGuard guard;
+        ret = hooks::realloc(ptr, size);
+    }
     if (ret) {
         if (ptr != nullptr) {
             tracking_api::Tracker::trackDeallocation(ptr, 0, hooks::Allocator::FREE);
@@ -220,8 +233,14 @@ void*
 mmap(void* addr, size_t length, int prot, int flags, int fd, off_t offset) noexcept
 {
     assert(hooks::mmap);
-    void* ptr = hooks::mmap(addr, length, prot, flags, fd, offset);
-    tracking_api::Tracker::trackAllocation(ptr, length, hooks::Allocator::MMAP);
+    void* ptr;
+    {
+        tracking_api::RecursionGuard guard;
+        ptr = hooks::mmap(addr, length, prot, flags, fd, offset);
+    }
+    if (ptr != MAP_FAILED) {
+        tracking_api::Tracker::trackAllocation(ptr, length, hooks::Allocator::MMAP);
+    }
     return ptr;
 }
 
@@ -230,8 +249,14 @@ void*
 mmap64(void* addr, size_t length, int prot, int flags, int fd, off64_t offset) noexcept
 {
     assert(hooks::mmap64);
-    void* ptr = hooks::mmap64(addr, length, prot, flags, fd, offset);
-    tracking_api::Tracker::trackAllocation(ptr, length, hooks::Allocator::MMAP);
+    void* ptr;
+    {
+        tracking_api::RecursionGuard guard;
+        ptr = hooks::mmap64(addr, length, prot, flags, fd, offset);
+    }
+    if (ptr != MAP_FAILED) {
+        tracking_api::Tracker::trackAllocation(ptr, length, hooks::Allocator::MMAP);
+    }
     return ptr;
 }
 #endif
@@ -241,7 +266,10 @@ munmap(void* addr, size_t length) noexcept
 {
     assert(hooks::munmap);
     tracking_api::Tracker::trackDeallocation(addr, length, hooks::Allocator::MUNMAP);
-    return hooks::munmap(addr, length);
+    {
+        tracking_api::RecursionGuard guard;
+        return hooks::munmap(addr, length);
+    }
 }
 
 void*
@@ -280,11 +308,16 @@ void*
 dlopen(const char* filename, int flag) noexcept
 {
     assert(hooks::dlopen);
-
-    void* ret = hooks::dlopen(filename, flag);
+    void* ret;
+    {
+        tracking_api::RecursionGuard guard;
+        ret = hooks::dlopen(filename, flag);
+    }
     if (ret) {
         tracking_api::Tracker::invalidate_module_cache();
-        if (filename && nullptr != strstr(filename, "/_greenlet.")) {
+        if (filename
+            && (nullptr != strstr(filename, "/_greenlet.") || nullptr != strstr(filename, "/greenlet.")))
+        {
             tracking_api::Tracker::beginTrackingGreenlets();
         }
     }
@@ -296,7 +329,11 @@ dlclose(void* handle) noexcept
 {
     assert(hooks::dlclose);
 
-    int ret = hooks::dlclose(handle);
+    int ret;
+    {
+        tracking_api::RecursionGuard guard;
+        ret = hooks::dlclose(handle);
+    }
     tracking_api::NativeTrace::flushCache();
     if (!ret) tracking_api::Tracker::invalidate_module_cache();
     return ret;
@@ -307,7 +344,11 @@ aligned_alloc(size_t alignment, size_t size) noexcept
 {
     assert(hooks::aligned_alloc);
 
-    void* ret = hooks::aligned_alloc(alignment, size);
+    void* ret;
+    {
+        tracking_api::RecursionGuard guard;
+        ret = hooks::aligned_alloc(alignment, size);
+    }
     if (ret) {
         tracking_api::Tracker::trackAllocation(ret, size, hooks::Allocator::ALIGNED_ALLOC);
     }
@@ -338,7 +379,11 @@ pvalloc(size_t size) noexcept
 {
     assert(hooks::pvalloc);
 
-    void* ret = hooks::pvalloc(size);
+    void* ret;
+    {
+        tracking_api::RecursionGuard guard;
+        ret = hooks::pvalloc(size);
+    }
     if (ret) {
         tracking_api::Tracker::trackAllocation(ret, size, hooks::Allocator::PVALLOC);
     }
